#!/usr/bin/env perl
#
# A script adding machines to LDAP for use by samba
#
# Authors: Odd Rune Dahle <oddrune@pvv.org> 
#          Svein Magne Bang <sveinmb@stud.ntnu.no>
#     
# Date:   2003-04-15


$DEBUG = 1;

$smbconf="/etc/samba/smb.conf";
$secretstdb="/var/lib/samba/secrets.tdb";
$tdbdumpbin="/usr/bin/tdbdump";
$machine_gid=70;

################# end of user configuration ##################
##############################################################


##
# First we need to read the ldap configuration from smb.conf
##

open(FILE,$smbconf) || die "Can't open $smbconf: $!";   # open smb.conf

$ldapdata[0] = "ldap server";
$ldapdata[1] = "ldap suffix";
$ldapdata[2] = "ldap admin dn";
$ldapdata[3] = "ldap rootpw";

while ($line = <FILE>) {     # parse file and extract ldap config
  if ($line =~ /ldap/i) {
   for($n=0;$n<3;$n++){
    if ($line =~ /$ldapdata[$n]/i) {
      $line =~ s/$ldapdata[$n]//i;
      $line =~ s/=//;
      $line =~ s/^\s+//;
      $line =~ s/"//g;
      $ldapdata[$n]=$line;
    }
   }
  }
}

for(n=0;$n<3;$n++){      # get rid of newlines
  chomp($ldapdata[$n]);
}

$ldapdata[1] =~ s/ou=people,//i;  # remove ou=people from basedn

close(FILE);   # close smb.conf



## 
# Now it's time to dump det secrets.tdb and retrive the ldap admin pw
##


$tdbret=`$tdbdumpbin $secretstdb`;  # execute tdbdump

$foundkey = "no";  # boolean used for loop

foreach $line (split ("\n", $tdbret)){  # parse tdbdump output
  if ($foundkey eq "yes"){   # if key is found, then extract rootwd
    $line =~ s/data = //i;
    $line =~ s/"//g;
    $ldapdata[3] = $line;
    last;
  } else {                   # still not found, keep searching
    if ($line =~ s/key = //i) {
      $line =~ s/"//g;
      $line =~ s/\//,/g;
      if($line eq $ldapdata[2]){ # see if this is the right key 
        $foundkey="yes";
      }
    }
  }
}  

# done reading configuration


$config{'server'} = $ldapdata[0];
$config{'basedn'} = $ldapdata[1];
$config{'rootdn'} = $ldapdata[2];
$config{'rootpw'} = $ldapdata[3];
$config{'machine_group'} = $machine_gid;


##
# Now it's time to actually add the machine
##


use Net::LDAP;
use Net::LDAP::Entry;


my $ldap;


sub ldap_connect($$$){
    my ($server) = @_;
    
    $ldap = Net::LDAP->new($server,version => '3') || error_msg("Server=$server");
    my $result = $ldap->start_tls(verify => 'none');
    die "LDAP Error: " . $result->error() if $result->code();
}

sub ldap_add_machine($$$$$$$$$){
    ($cn, $uid, $userpw, $uidNumber, $gidNumber, 
     $rootpw, $rootdn, $basedn, $homedir, $maildir, $shell) = @_;

    $ldap->bind($rootdn, password => $rootpw);
    my $entry = Net::LDAP::Entry->new();
    $entry->dn("uid=$uid,ou=People,$basedn");
    $entry->add(
		objectclass => ['posixAccount'],
		cn => $cn,
		uid => $uid,
		uidNumber => $uidNumber,
		gidNumber => $gidNumber,
		homeDirectory => $homedir,
		userPassword => "{crypt}". $userpw,
		loginShell => $shell,
		);
    
    return $entry->update($ldap);
}

sub ldap_add_group($$$$$){
    ($uid, $gidNumber, $rootpw, $rootdn, $basedn) = @_;

    $ldap->bind($rootdn, password => $rootpw);
    my $entry = Net::LDAP::Entry->new();
    $entry->dn("cn=$uid,ou=Group,$basedn");
    $entry->add(
                objectclass => 'posixGroup',
                cn => $uid,
                gidNumber => $gidNumber,
                );

    return $entry->update($ldap);
}

sub ldap_close(){
    $ldap->unbind();
}

sub ldap_get_max_uid_for_workstations($){
    my ($basedn) = @_;
    my $mesg = $ldap->search (base   => "ou=People,$basedn",
			      filter => "objectClass=posixAccount"
                              );
    my $minval = 7000;			      
    my $maxval = 9000;  # maxval must be below 10000 to avoid conflicts
    			# with the skolelinux user-range.
    foreach $entry ($mesg->all_entries()){
	my $val = $entry->get_value('uidNumber');
	$minval = $val if ($val > $minval) and ($val < $maxval);
    }
    return ++$minval;
}

sub ldap_machine_exists($$$){
    my ($uidNumber, $basedn, $filtername) = @_;

    my $mesg = $ldap->search (base   => "ou=People,$basedn",
			      filter => "$filtername=$uidNumber"
                              );
    return $mesg->count();
}


if (!$ARGV[0]) {
	print "Usage: smbaddclient <clientname\$>\n";
	exit 2;
} 

my $uid = lc($ARGV[0]);  # lowercase

&ldap_connect($config{'server'}, $config{'rootdn'});

my $uidnumber= &ldap_get_max_uid_for_workstations($config{'basedn'});


# This should be a unneccessary test, but I want it here
# anyway -- prevents a lot of mess if there are bugs elsewhere.
my $count = &ldap_machine_exists($uidnumber, $config{'basedn'}, 'uidNumber');
if ($count) {
	print "[debug] uidNumber $uidnumber already exists.\n" if $DEBUG;
	exit 1;
}

$count = &ldap_machine_exists($uid, $config{'basedn'}, 'uid');
if ($count) {
	print "[debug] uid $uid already exists.\n" if $DEBUG;
	exit 1;
}

# make sure the machines-group exists - ugly way to do it, but it works :)
&ldap_add_group('machines',$config{'machine_group'}, $config{'rootpw'}, 
	$config{'rootdn'}, $config{'basedn'});

$result = &ldap_add_machine("Machine $uid",$uid,'*',$uidnumber,
		$config{'machine_group'},$config{'rootpw'},$config{'rootdn'},
		$config{'basedn'},'/var','/tmp','/bin/false');
if ($result->code()) {
	print "[error] $result \n";
} else {
	print "Successfully added client $uid.\n";
}

&ldap_close();
